@@ -13,8 +13,6 @@
 /* own header files */
 #include "session.h"
 
-#define CID_MAX_ROWS 100
-
 enum call_type_t {
   CALL_IN,
   CALL_OUT
@@ -26,3 +24,6 @@ void cid_add_line(struct session_t *session,
 		  enum call_type_t ct, gchar *from, gchar *to);
 void cid_add_data(struct session_t *session, gchar *from);
 void cid_call_finished(struct session_t *session, gchar *message);
+void cid_normalize(struct session_t *session);
+void cid_add_saved_line(struct session_t *session, char *date, char *type,
+			char *from, char *to, char *duration);
