@@ -44,6 +44,15 @@ static int gtksettings_try(GtkWidget *widget) {
   GtkWidget *entry;
   GtkWidget *button;
 
+  /* exec_on_incoming */
+  entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget),
+					    "exec_on_incoming_entry");
+  if (entry) {
+    free(session->exec_on_incoming);
+    session->exec_on_incoming = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
+  } else
+    fprintf(stderr, "gtksettings_cb_ok: Error getting exec_on_incoming.\n");
+
   /* msn */
   entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget), "msn_entry");
   if (entry)
@@ -68,6 +77,16 @@ static int gtksettings_try(GtkWidget *widget) {
     fprintf(stderr, "gtksettings_cb_ok: Error getting history.\n");
   session->dial_number_history_pointer = 0;
 
+  /* cid_max_entry */
+  entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget),
+					    "cid_max_entry");
+  if (entry)
+    session->cid_num_max =
+      strtol(gtk_entry_get_text(GTK_ENTRY(entry)), NULL, 0);
+  else
+    fprintf(stderr, "gtksettings_cb_ok: "
+	    "Error getting caller id maximum rows.\n");
+
   /* save checkbutton */
   button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(widget),
 					     "save_checkbutton");
@@ -81,12 +100,11 @@ static int gtksettings_try(GtkWidget *widget) {
   /*
    * audio stop/start
    */
-  if (session->audio_device_name_in) { /* not shut down */
+
+  if (session->audio_device_name_in) { /* shut down if defined */
     session_io_handlers_stop(session);
     if (!session->option_release_devices) /* audio_close if normal mode */
-      if (session_audio_close(session))
-	fprintf(stderr, "Error closing audio device.\n");
-    session_audio_free(session);
+      session_audio_deinit(session);
     free(session->audio_device_name_in);
     free(session->audio_device_name_out);
   }
@@ -120,21 +138,19 @@ static int gtksettings_try(GtkWidget *widget) {
     fprintf(stderr,
 	    "gtksettings_cb_ok: Error getting release_devices state.\n");
 
-  if (session_audio_init(session)) {
-    successful = 0;
-    free(session->audio_device_name_in);
-    free(session->audio_device_name_out);
-    session->audio_device_name_in = NULL; /* flag for audio shut down */
+  if (!session->option_release_devices) {
+    if (session_audio_init(session)) {
+      successful = 0;
+      free(session->audio_device_name_in);
+      free(session->audio_device_name_out);
+      session->audio_device_name_in = NULL; /* flag for audio shut down */
+    }
   }
 
-  if (session->audio_device_name_in) { /* not shut down */
-    if (session->option_release_devices) /* audio_close if release mode */
-      if (session_audio_close(session))
-	fprintf(stderr, "Error closing audio device.\n");
+  if (session->audio_device_name_in) { /* OK or unknown */
     session_io_handlers_start(session);
     session_set_state(session, STATE_READY); /* update everything */
   }
-  
 
   /* try to apply msn settings */
   if (isdn_setMSN(session->isdn_fd, session->msn) ||
@@ -167,7 +183,7 @@ static void gtksettings_cb_ok(GtkWidget *window) {
     gtk_widget_destroy(window);
   } else { /* settings error -> restored old settings -> display note */
     ok_window = ok_dialog_get("Note",
-			   "Bad sound/isdn device settings, please try again.",
+			   "Bad isdn/sound device settings, please try again.",
 			   GTK_JUSTIFY_CENTER);
     gtk_window_set_modal(GTK_WINDOW(ok_window), TRUE);
     gtk_widget_show(ok_window);
@@ -206,9 +222,11 @@ void gtksettings_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   GtkWidget *msn_entry;                  /* phone page */
   GtkWidget *msns_entry;
   GtkWidget *history_entry;
+  GtkWidget *cid_max_entry;
   GtkWidget *audio_device_name_in_entry; /* sound devices page */
   GtkWidget *audio_device_name_out_entry;
   GtkWidget *release_checkbutton; 
+  GtkWidget *exec_on_incoming_entry; 
 
   char *s; /* temporary string */
 
@@ -233,12 +251,30 @@ void gtksettings_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show(frame);
 
+  table = gtk_table_new(2, 2, FALSE); /* rows, columns, not homogeneous */
+  gtk_container_add(GTK_CONTAINER(frame), table);
+  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+  gtk_widget_show(table);
+
   save_checkbutton = gtk_check_button_new_with_label("Save options on exit");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(save_checkbutton),
 			       session->option_save_options);
-  gtk_container_add(GTK_CONTAINER(frame), save_checkbutton);
+  gtk_table_attach_defaults(GTK_TABLE(table), save_checkbutton, 0,2,0,1);
   gtk_widget_show(save_checkbutton);
 
+  label = gtk_label_new("Execute on incoming call:");
+  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
+  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+  gtk_widget_show(label);
+
+  exec_on_incoming_entry = gtk_entry_new();
+  gtk_table_attach_defaults(GTK_TABLE(table), exec_on_incoming_entry, 1,2,1,2);
+  gtk_entry_set_text(GTK_ENTRY(exec_on_incoming_entry),
+		     session->exec_on_incoming);
+  gtk_widget_show(exec_on_incoming_entry);
+
   /* Phone page */
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox,
@@ -302,6 +338,34 @@ void gtksettings_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
   free(s);
   gtk_widget_show(history_entry);
 
+  frame = gtk_frame_new("Caller ID");
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
+  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+  gtk_widget_show(frame);
+
+  table = gtk_table_new(1, 2, FALSE); /* rows, columns, not homogeneous */
+  gtk_container_add(GTK_CONTAINER(frame), table);
+  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+  gtk_widget_show(table);
+
+  label = gtk_label_new("Maximum CID rows:");
+  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,0,1);
+  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+  gtk_widget_show(label);
+
+  cid_max_entry = gtk_entry_new();
+  gtk_table_attach_defaults(GTK_TABLE(table), cid_max_entry, 1,2,0,1);
+  if (session->cid_num_max == 0) {
+    s = strdup("[no limit]");
+  } else {
+    asprintf(&s, "%u", session->cid_num_max);
+  }
+  gtk_entry_set_text(GTK_ENTRY(cid_max_entry), s);
+  free(s);
+  gtk_widget_show(cid_max_entry);
+
   /* Sound devices page */
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox,
@@ -364,11 +428,15 @@ void gtksettings_cb(GtkWidget *widget _U_, gpointer data, guint action _U_) {
 
   gtk_object_set_data(GTK_OBJECT(window), "save_checkbutton",
 		      (gpointer) save_checkbutton);
+  gtk_object_set_data(GTK_OBJECT(window), "exec_on_incoming_entry",
+		      (gpointer) exec_on_incoming_entry);
 
   gtk_object_set_data(GTK_OBJECT(window), "msn_entry", (gpointer) msn_entry);
   gtk_object_set_data(GTK_OBJECT(window), "msns_entry", (gpointer) msns_entry);
   gtk_object_set_data(GTK_OBJECT(window), "history_entry",
 		      (gpointer) history_entry);
+  gtk_object_set_data(GTK_OBJECT(window), "cid_max_entry",
+		      (gpointer) cid_max_entry);
 
   gtk_object_set_data(GTK_OBJECT(window), "audio_device_name_in_entry",
 		      (gpointer) audio_device_name_in_entry);
