@@ -7,7 +7,11 @@
  *
  */
 
-#include <termios.h>
+#include "config.h"
+
+#ifdef HAVE_TERMIOS_H
+  #include <termios.h>
+#endif
 
 #define LOCK_PATH "/var/lock"
 
