@@ -27,7 +27,7 @@ enum state_t {
   STATE_RINGING_QUIET,  /* same as above, audio off (device blocked) */
   STATE_DIALING,        /* we are dialing out */
   STATE_CONVERSATION,   /* we are talking */
-  STATE_SERVICE,        /* special mode (init, llcheck, ...) */
+  STATE_SERVICE,        /* special mode (llcheck) */
 
   STATE_LAST /* dummy to calculate size */
 };
@@ -137,6 +137,7 @@ struct session_t {
   GtkWidget *cid_list; /* the list to hold the individual call data */
   GtkWidget *cid_scrolled_window; /* the home of the clist with adjustments */
   gint cid_num; /* number of rows in list */
+  gint cid_num_max; /* maximum number of rows in list */
   time_t vcon_time; /* the start of conversation mode */
   /* the symbols for the CList */
   GdkPixmap *symbol_in_pixmap;
@@ -170,6 +171,11 @@ struct session_t {
   int option_show_callerid;   /* show callerid part in main window */
   int option_show_controlpad; /* show control pad (key pad etc.) */
   int option_muted;           /* mute microphone (other party gets zeros) */
+
+  char *exec_on_incoming; /* string with command to execute on incoming call */
+
+  int local_sock; /* unix domain socket for local server functionality */
+  gint gdk_local_input_tag; /* gdk tag for gtk main loop select */
 };
 
 extern struct session_t session;
@@ -197,6 +203,7 @@ void gdk_handle_isdn_input(gpointer data, gint fd,
 			   GdkInputCondition condition);
 void gdk_handle_audio_input(gpointer data, gint fd,
 			   GdkInputCondition condition);
+void session_make_call(struct session_t *session, char *number);
 void gtk_handle_pick_up_button(GtkWidget *widget, gpointer data);
 void gtk_handle_hang_up_button(GtkWidget *widget, gpointer data);
 
